@@ -12,6 +12,7 @@ sub new {
 
     my $self = {
         name      => "unknown name",
+        umask     => undef,
         autoflush => 1,
         mode      => "append",
         @options,
@@ -43,6 +44,8 @@ sub file_open {
 
     my $arrows = ">";
 
+    my $old_umask = umask();
+
     if($self->{mode} eq "append") {
         $arrows = ">>";
     } elsif ($self->{mode} eq "pipe") {
@@ -51,8 +54,12 @@ sub file_open {
 
     my $fh = do { local *FH; *FH; };
 
+    umask($self->{umask}) if defined $self->{umask};
+
     open $fh, "$arrows$self->{filename}" or
-        die "Can't open $self->{filename} ($@)";
+        die "Can't open $self->{filename} ($!)";
+
+    umask($old_umask) if defined $self->{umask};
 
     $self->{fh} = $fh;
 
@@ -118,6 +125,7 @@ Log::Log4perl::Appender::File - Log to file
       filename  => 'file.log',
       mode      => 'append',
       autoflush => 1,
+      umask     => 0222,
     );
 
     $file->log(message => "Log me\n");
@@ -126,15 +134,6 @@ Log::Log4perl::Appender::File - Log to file
 
 This is a simple appender for writing to a file.
 
-The constructor C<new()> opens a file, specified in C<filename>, for
-writing. If C<mode> is C<append>, it will append to the file if it
-exists, if it is C<pipe> the file will be understood as executable
-to pipe output to. On other settings of C<mode> it will clobber any existing
-file first and then start writing to it. The default C<mode> is C<append>.
-
-C<autoflush>, if set to a true value, triggers flushing the data
-out to the file on every call to C<log()>.
-
 The C<log()> method takes a single scalar. If a newline character
 should terminate the message, it has to be added explicitely.
 
@@ -142,9 +141,39 @@ Upon destruction of the object, the filehandle to access the
 file is flushed and closed.
 
 If you want to switch over to a different logfile, use the
-C<switch_file($newfile)> method which will first close the old
+C<file_switch($newfile)> method which will first close the old
 file handle and then open a one to the new file specified.
 
+=head2 OPTIONS
+
+=over 4
+
+=item filename
+
+Name of the log file.
+
+=item mode
+
+Messages will be append to the file if C<$mode> is set to the
+string C<"append">. Will clobber the file
+if set to C<"clobber">. If it is C<"pipe">, the file will be understood 
+as executable to pipe output to. Default mode is C<"append">.
+
+=item autoflush
+
+C<autoflush>, if set to a true value, triggers flushing the data
+out to the file on every call to C<log()>. C<autoflush> is on by default.
+
+=item umask
+
+Specifies the C<umask> to use when creating the file, determining
+the file's permission settings. 
+If set to C<0222> (default), new
+files will be created with C<rw-r--r--> permissions.
+If set to C<0000>, new files will be created with C<rw-rw-rw-> permissions.
+
+=back
+
 Design and implementation of this module has been greatly inspired by
 Dave Rolsky's C<Log::Dispatch> appender framework.
 
