@@ -34,6 +34,7 @@ my @LEVEL_MAP_A = qw(
 
 our $WATCHER;
 our $DEFAULT_WATCH_DELAY = 60; # seconds
+our $OLD_CONFIG;
 
 ###########################################
 sub init {
@@ -84,13 +85,20 @@ sub init_and_watch {
                    );
     }
 
-    _init($class, $config);
+    eval { _init($class, $config); };
+
+    if($@) {
+        die "$@" unless defined $OLD_CONFIG;
+            # Call _init with a pre-parsed config to go back to old setting
+        _init($class, undef, $OLD_CONFIG);
+        warn "Loading new config failed, reverted to old one\n";
+    }
 }
 
 ##################################################
 sub _init {
 ##################################################
-    my($class, $config) = @_;
+    my($class, $config, $data) = @_;
 
     my %additivity = ();
 
@@ -118,7 +126,7 @@ sub _init {
     # Pretty scary. But it allows the lines of the config file to be
     # in *arbitrary* order.
 
-    my $data = config_read($config);
+    $data = config_read($config) unless defined $data;
     
     if(_INTERNAL_DEBUG) {
         require Data::Dumper;
@@ -179,6 +187,11 @@ sub _init {
                     print "Found entry @$path\n" if _INTERNAL_DEBUG;
                     $filter_names{@$path[0]}++;
                 } else {
+
+                    if (ref($value) eq "ARRAY") {
+                      die "Multiple definitions of logger ".join('.',@$path)." in log4perl config";
+                    }
+
                     # This is a regular logger
                     push @loggers, [join('.', @$path), $value];
                 }
@@ -280,6 +293,9 @@ sub _init {
         warn "Log::Log4perl configuration looks suspicious: ",
              "$CONFIG_INTEGRITY_ERROR";
     }
+
+        # Successful init(), save config for later
+    $OLD_CONFIG = $data;
 }
 
 ##################################################
@@ -522,6 +538,11 @@ sub config_read {
     } elsif (ref $config eq 'SCALAR') {
         @text = split(/\n/,$$config);
 
+    } elsif (ref $config eq 'GLOB' or 
+             ref $config eq 'IO::File') {
+            # If we have a file handle, just call the reader
+        config_file_read($config, \@text);
+
     } elsif (ref $config) {
             # Caller provided a config parser object, which already
             # knows which file (or DB or whatever) to parse.
@@ -569,12 +590,7 @@ sub config_read {
             }
         }else{
             open FILE, "<$config" or die "Cannot open config file '$config'";
-            {
-                   # Dennis Gregorovic <dgregor@redhat.com> added this
-                   # to protect apps which are tinkering with $/ globally.
-               local $/ = "\n";
-               @text = <FILE>;
-            }
+            config_file_read(\*FILE, \@text);
             close FILE;
         }
     }
@@ -600,6 +616,19 @@ sub config_read {
     return $data;
 }
 
+
+###########################################
+sub config_file_read {
+###########################################
+    my($handle, $linesref) = @_;
+
+        # Dennis Gregorovic <dgregor@redhat.com> added this
+        # to protect apps which are tinkering with $/ globally.
+    local $/ = "\n";
+
+    @$linesref = <$handle>;
+}
+
 ###########################################
 sub unlog4j {
 ###########################################
